/****************************************************************************/
/*                                                                          */
/* For a draft on the risk semantics, please look at doc/theory/risks       */
/*                                                                          */
/****************************************************************************/

#include <stdlib.h>

#include <idsa_internal.h>

unsigned int idsa_risk_parse(const char *s)
{
  double p, c;
  int i = 0;

  p = atof(s);

  while ((s[i] != '\0') && (s[i] != '/')) {
    i++;
  }

  if (s[i] == '/') {
    i++;
    c = atof(s + i);
  } else {
    c = 1.0;
  }

  return idsa_risk_make(p, c);
}

unsigned int idsa_risk_make(double severity, double confidence)
{
  signed short s;
  unsigned short c;

  s = (signed short) (severity * 32767);
  c = (unsigned short) ((1.0 - confidence) * 65535);

  return ((s & 0xffff) << 16) | (c & 0xffff);
}

double idsa_risk_severity(unsigned int risk)
{
  return (double) (risk >> 16) / 32767.0;
}

double idsa_risk_confidence(unsigned int risk)
{
  return 1.0 - (double) (risk & 0xffff) / 65535.0;
}
